#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright © 2014 by James Conrad Shea (Duckstab).
#
# Utility methods to support ^nav^ unit naming convention, requires compatible version of Unit Naming

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from CvPythonExtensions import *

import CvUtil
import sys
import math
import string
import RandomNameUtils
import BugUtil
import random
import BugData
import os
import NavalUnitLists
import re

gc = CyGlobalContext()
gRandom = random.Random()

def getDataValue(key):
    counters = BugData.getGameData().getTable("NavalUnitNaming")
    if (not counters.hasTable(key)):
        counter = counters.getTable(key)
        result = None
    else:
        counter = counters.getTable(key)
        result = counter["val"]
    return result

def setDataValue(key, value):
    BugUtil.debug("setDataValue(%s, %s)" % (key, value))
    counters = BugData.getGameData().getTable("NavalUnitNaming")
    counter = counters.getTable(key)
    counter["val"] = value
    BugData.save()


def getRandom():
    seed = getDataValue("seed")
    if (seed is None):
        seed = gRandom.random()
        setDataValue("seed", seed)
        gRandom.seed(seed)
    return gRandom

class Namer:

    def __init__(self, prefix):
        self.prefix = prefix

    def generate(self, iCivilization, unitClass, count):
        return RandomNameUtils.getRandomCivilizationName(iCivilization)

    def generateWithPrefix(self, prefixOverride, iCivilization, unitClass, count):
        pfx = self.prefix
        if (prefixOverride is not None):
            pfx = prefixOverride
        if (pfx == ""):
            pfx = None
        if (pfx is not None):
            return pfx + " " + self.generate(iCivilization, unitClass, count)
        else:
            return self.generate(iCivilization, unitClass, count)

class ListNamer(Namer):

    def __init__(self, key, prefix, nameList):
        Namer.__init__(self, prefix)
        self.nameList = nameList
        self.key = key
        # Defer the rest of initialization until BugData has fully loaded and had its event handlers called; otherwise data does not persist across saves
        self.fullyInitialized = False        

    def _deferredInit(self):
        lst = getDataValue(self.key)
        if (lst is None):
            seed = gRandom.random()
            lst = [seed, 0]
            setDataValue(self.key, lst)
        else:
            seed = lst[0]
        rnd = random.Random()
        rnd.seed(seed)
        rnd.shuffle(self.nameList)
        self.fullyInitialized = True

    def _nextCount(self):
        lst = getDataValue(self.key)
        result = lst[1]
        lst[1] = result + 1
        setDataValue(self.key, lst)
        return result   

    def generate(self, iCivilization, unitClass, count):
        if (not self.fullyInitialized):
            self._deferredInit()
        index = self._nextCount()
        if (index < len(self.nameList)):
            return self.nameList[index]
        else:
            return Namer.generate(self, iCivilization, unitClass, count)

class IncreasingSequenceNamer(Namer):

    def __init__(self, key, prefix, classIncrement, modulus):
        Namer.__init__(self, prefix)
        self.key = key
        self.classIncrement = classIncrement
        self.modulus = modulus
        # Defer the rest of initialization until BugData has fully loaded and had its event handlers called; otherwise data does not persist across saves
        self.fullyInitialized = False

    def _deferredInit(self):
        lst = getDataValue(self.key)
        if (lst is None):
            self.lastUnitClass = None
            self.lastValue = 0
            lst = [self.lastUnitClass, self.lastValue]
            setDataValue(self.key, lst)
        else:
            self.lastUnitClass = lst[0]
            self.lastValue = lst[1]
        self.fullyInitialized = True
            
    def generate(self, iCivilization, unitClass, count):
        if (not self.fullyInitialized):
            self._deferredInit()
        if ((self.lastValue == 0) or (unitClass == self.lastUnitClass)):
            self.lastValue += gRandom.randint(1, self.modulus)
        else:
            self.lastValue = (self.classIncrement * (1 + int(self.lastValue / self.classIncrement))) + gRandom.randint(1, self.modulus)
        self.lastUnitClass = unitClass
        result = "%s%d" % (self.prefix, self.lastValue)
        setDataValue(self.key, (self.lastUnitClass, self.lastValue))
        return result
    
    def generateWithPrefix(self, prefixOverride, iCivilization, unitClass, count):
        return self.generate(iCivilization, unitClass, count)

AMERICAN_SHARED_NAMER_1 = ListNamer("Am1", NavalUnitLists.AMERICAN_SHIP_PREFIX, NavalUnitLists.AMERICAN_FRIGATES)
AMERICAN_SHARED_NAMER_2 = ListNamer("Am2", NavalUnitLists.AMERICAN_SHIP_PREFIX, NavalUnitLists.AMERICAN_TRANSPORTS)

CHINESE_SHARED_NAMER_1 = ListNamer("Ch1", NavalUnitLists.CHINESE_SHIP_PREFIX, NavalUnitLists.CHINESE_FRIGATES)
CHINESE_SHARED_NAMER_2 = ListNamer("Ch2", NavalUnitLists.CHINESE_SHIP_PREFIX, NavalUnitLists.CHINESE_TRANSPORTS)
CHINESE_SHARED_NAMER_3 = ListNamer("Ch3", NavalUnitLists.CHINESE_SHIP_PREFIX, NavalUnitLists.CHINESE_DESTROYERS)

DUTCH_SHARED_NAMER_1 = ListNamer("Du1", NavalUnitLists.DUTCH_SHIP_PREFIX, NavalUnitLists.DUTCH_FRIGATES)
DUTCH_SHARED_NAMER_2 = ListNamer("Du2", NavalUnitLists.DUTCH_SHIP_PREFIX, NavalUnitLists.DUTCH_TRANSPORTS)
DUTCH_SHARED_NAMER_3 = ListNamer("Du3", NavalUnitLists.DUTCH_SHIP_PREFIX, NavalUnitLists.DUTCH_ATTACK_SUBMARINES)

ENGLISH_SHARED_NAMER_1 = ListNamer("En1", NavalUnitLists.ENGLISH_SHIP_PREFIX, NavalUnitLists.ENGLISH_GALLEONS)
ENGLISH_SHARED_NAMER_2 = ListNamer("En2", NavalUnitLists.ENGLISH_SHIP_PREFIX, NavalUnitLists.ENGLISH_DESTROYERS)
ENGLISH_SHARED_NAMER_3 = ListNamer("En3", NavalUnitLists.ENGLISH_SHIP_PREFIX, NavalUnitLists.ENGLISH_CARAVELS)

FRENCH_SHARED_NAMER_1 = ListNamer("Fr1", None, NavalUnitLists.FRENCH_FRIGATES)

GERMAN_SUBMARINE_NAMER = IncreasingSequenceNamer("GeU", NavalUnitLists.GERMAN_SUBMARINE_PREFIX, 70, 5)
GERMAN_SHARED_NAMER_1 = ListNamer("Ge1", NavalUnitLists.GERMAN_SHIP_PREFIX, NavalUnitLists.GERMAN_DESTROYERS)
GERMAN_SHARED_NAMER_2 = ListNamer("Ge2", NavalUnitLists.GERMAN_SHIP_PREFIX, NavalUnitLists.GERMAN_GALLEONS)

GREEK_SHARED_NAMER_1 = ListNamer("Gr1", NavalUnitLists.GREEK_SHIP_PREFIX, NavalUnitLists.GREEK_DESTROYERS)
GREEK_SHARED_NAMER_2 = ListNamer("Gr2", NavalUnitLists.GREEK_SHIP_PREFIX, NavalUnitLists.GREEK_ATTACK_SUBMARINES)
GREEK_SHARED_NAMER_3 = ListNamer("Gr3", NavalUnitLists.GREEK_SHIP_PREFIX, NavalUnitLists.GREEK_TRANSPORTS)

INDIAN_SHARED_NAMER_1 = ListNamer("In1", NavalUnitLists.INDIAN_SHIP_PREFIX, NavalUnitLists.INDIAN_DESTROYERS)
INDIAN_SHARED_NAMER_2 = ListNamer("In2", NavalUnitLists.INDIAN_SHIP_PREFIX, NavalUnitLists.INDIAN_TRANSPORTS)
INDIAN_SHARED_NAMER_3 = ListNamer("In3", NavalUnitLists.INDIAN_SHIP_PREFIX, NavalUnitLists.INDIAN_ATTACK_SUBMARINES)

JAPANESE_SHARED_NAMER_1 = ListNamer("Jp1", None, NavalUnitLists.JAPANESE_FRIGATES)
JAPANESE_SHARED_NAMER_2 = ListNamer("Jp2", None, NavalUnitLists.JAPANESE_TRANSPORTS)
JAPANESE_SHARED_NAMER_3 = ListNamer("Jp3", None, NavalUnitLists.JAPANESE_DESTROYERS)
JAPANESE_SUBMARINE_NAMER = IncreasingSequenceNamer("JaS", NavalUnitLists.JAPANESE_SUBMARINE_PREFIX, 100, 10)

KOREAN_SHARED_NAMER_1 = ListNamer("Kr1", NavalUnitLists.KOREAN_SHIP_PREFIX, NavalUnitLists.KOREAN_SUBMARINES)
KOREAN_SHARED_NAMER_2 = ListNamer("Kr2", NavalUnitLists.KOREAN_SHIP_PREFIX, NavalUnitLists.KOREAN_DESTROYERS)
KOREAN_SHARED_NAMER_3 = ListNamer("Kr3", NavalUnitLists.KOREAN_SHIP_PREFIX, NavalUnitLists.KOREAN_TRANSPORTS)

OTTOMAN_SHARED_NAMER_1 = ListNamer("Ot1", NavalUnitLists.OTTOMAN_SHIP_PREFIX, NavalUnitLists.OTTOMAN_FRIGATES)
OTTOMAN_SHARED_NAMER_2 = ListNamer("Ot2", NavalUnitLists.OTTOMAN_SHIP_PREFIX, NavalUnitLists.OTTOMAN_TRANSPORTS)

PERSIAN_SHARED_NAMER_1 = ListNamer("Pr1", None, NavalUnitLists.PERSIAN_DESTROYERS)
PERSIAN_SHARED_NAMER_2 = ListNamer("Pr2", None, NavalUnitLists.PERSIAN_TRANSPORTS)

PORTUGUESE_SHARED_NAMER_1 = ListNamer("Po1", NavalUnitLists.PORTUGUESE_SHIP_PREFIX, NavalUnitLists.PORTUGUESE_GALLEONS)
PORTUGUESE_SHARED_NAMER_2 = ListNamer("Po2", NavalUnitLists.PORTUGUESE_SHIP_PREFIX, NavalUnitLists.PORTUGUESE_CARRACKS)

RUSSIAN_SHARED_NAMER_1 = ListNamer("Ru1", None, NavalUnitLists.RUSSIAN_FRIGATES)
RUSSIAN_SHARED_NAMER_2 = ListNamer("Ru2", None, NavalUnitLists.RUSSIAN_TRANSPORTS)

SPANISH_SHARED_NAMER_1 = ListNamer("Sp1", None, NavalUnitLists.SPANISH_GALLEONS)
SPANISH_SHARED_NAMER_2 = ListNamer("Sp2", None, NavalUnitLists.SPANISH_CARAVELS)

ATTACK_SUBMARINE = "Attack Submarine"
BATTLESHIP = "Battleship"
CARAVEL = "Caravel"
CARRIER = "Carrier"
DESTROYER = "Destroyer"
FRIGATE = "Frigate"
GALLEON = "Galleon"
GALLEY = "Galley"
IRONCLAD = "Ironclad"
MISSILE_CRUISER = "Missile Cruiser"
PRIVATEER = "Privateer"
SHIP_OF_THE_LINE = "Ship of the Line"
STEALTH_DESTROYER = "Stealth Destroyer"
SUBMARINE = "Submarine"
TRANSPORT = "Transport"
TRIREME = "Trireme"
DEFAULT = "DEFAULT"

NAMER_MAP = {
    "CIVILIZATION_AMERICA": {
        ATTACK_SUBMARINE: ListNamer("AmA", NavalUnitLists.AMERICAN_SHIP_PREFIX, NavalUnitLists.AMERICAN_ATTACK_SUBMARINES),
        BATTLESHIP: ListNamer("AmB", NavalUnitLists.AMERICAN_SHIP_PREFIX, NavalUnitLists.AMERICAN_BATTLESHIPS),
        CARAVEL: AMERICAN_SHARED_NAMER_1,
        CARRIER: ListNamer("AmC", NavalUnitLists.AMERICAN_SHIP_PREFIX, NavalUnitLists.AMERICAN_CARRIERS),
        DESTROYER: ListNamer("AmD", NavalUnitLists.AMERICAN_SHIP_PREFIX, NavalUnitLists.AMERICAN_DESTROYERS),
        FRIGATE: AMERICAN_SHARED_NAMER_1,
        GALLEON: AMERICAN_SHARED_NAMER_2,
        GALLEY: AMERICAN_SHARED_NAMER_2,
        IRONCLAD: ListNamer("AmI", NavalUnitLists.AMERICAN_SHIP_PREFIX, NavalUnitLists.AMERICAN_IRONCLADS),
        MISSILE_CRUISER: ListNamer("AmM", NavalUnitLists.AMERICAN_SHIP_PREFIX, NavalUnitLists.AMERICAN_MISSILE_CRUISERS),
        PRIVATEER: ListNamer("AmP", None, NavalUnitLists.AMERICAN_PRIVATEERS),
        SHIP_OF_THE_LINE: ListNamer("AmSl", NavalUnitLists.AMERICAN_SHIP_PREFIX, NavalUnitLists.AMERICAN_SHIPS_OF_THE_LINE),
        STEALTH_DESTROYER: ListNamer("AmSd", NavalUnitLists.AMERICAN_SHIP_PREFIX, NavalUnitLists.AMERICAN_STEALTH_DESTROYERS),
        SUBMARINE: ListNamer("AmSu", NavalUnitLists.AMERICAN_SHIP_PREFIX, NavalUnitLists.AMERICAN_SUBMARINES),
        TRANSPORT: AMERICAN_SHARED_NAMER_2,
        TRIREME: AMERICAN_SHARED_NAMER_1,
        DEFAULT: Namer(NavalUnitLists.AMERICAN_SHIP_PREFIX)
        },
    "CIVILIZATION_ARABIA": {
        DEFAULT: Namer(NavalUnitLists.ARABIAN_SHIP_PREFIX)
        },
    "CIVILIZATION_CHINA": {
        ATTACK_SUBMARINE: ListNamer("ChA", NavalUnitLists.CHINESE_SHIP_PREFIX, NavalUnitLists.CHINESE_ATTACK_SUBMARINES),
        BATTLESHIP: ListNamer("ChB", NavalUnitLists.CHINESE_SHIP_PREFIX, NavalUnitLists.CHINESE_BATTLESHIPS),
        CARAVEL: CHINESE_SHARED_NAMER_1,
        DESTROYER: CHINESE_SHARED_NAMER_3,
        FRIGATE: CHINESE_SHARED_NAMER_1,
        GALLEON: CHINESE_SHARED_NAMER_2,
        GALLEY: CHINESE_SHARED_NAMER_2,
        IRONCLAD: ListNamer("ChI", NavalUnitLists.CHINESE_SHIP_PREFIX, NavalUnitLists.CHINESE_IRONCLADS),
        SHIP_OF_THE_LINE: CHINESE_SHARED_NAMER_3,
        TRANSPORT: CHINESE_SHARED_NAMER_2,
        TRIREME: CHINESE_SHARED_NAMER_1,
        DEFAULT: Namer(NavalUnitLists.CHINESE_SHIP_PREFIX)
        },
    "CIVILIZATION_ENGLAND": {
        ATTACK_SUBMARINE: ListNamer("EnA", NavalUnitLists.ENGLISH_SHIP_PREFIX, NavalUnitLists.ENGLISH_ATTACK_SUBMARINES),
        BATTLESHIP: ListNamer("EnB", NavalUnitLists.ENGLISH_SHIP_PREFIX, NavalUnitLists.ENGLISH_BATTLESHIPS),
        CARAVEL: ENGLISH_SHARED_NAMER_3,
        CARRIER: ListNamer("EnC", NavalUnitLists.ENGLISH_SHIP_PREFIX, NavalUnitLists.ENGLISH_CARRIERS),
        DESTROYER: ENGLISH_SHARED_NAMER_2,
        FRIGATE: ListNamer("EnF", NavalUnitLists.ENGLISH_SHIP_PREFIX, NavalUnitLists.ENGLISH_FRIGATES),
        GALLEON: ENGLISH_SHARED_NAMER_1,
        GALLEY: ENGLISH_SHARED_NAMER_1,
        IRONCLAD: ENGLISH_SHARED_NAMER_2,
        PRIVATEER: ListNamer("EnP", None, NavalUnitLists.ENGLISH_PRIVATEERS),
        SHIP_OF_THE_LINE: ListNamer("EnSl", NavalUnitLists.ENGLISH_SHIP_PREFIX, NavalUnitLists.ENGLISH_SHIPS_OF_THE_LINE),
        STEALTH_DESTROYER: ListNamer("EnSd", NavalUnitLists.ENGLISH_SHIP_PREFIX, NavalUnitLists.ENGLISH_STEALTH_DESTROYERS),
        SUBMARINE: ListNamer("EnSu", NavalUnitLists.ENGLISH_SHIP_PREFIX, NavalUnitLists.ENGLISH_SUBMARINES),
        TRANSPORT: ListNamer("EnT", NavalUnitLists.ENGLISH_AUXILIARY_PREFIX, NavalUnitLists.ENGLISH_TRANSPORTS),
        TRIREME: ENGLISH_SHARED_NAMER_3,
        DEFAULT: Namer(NavalUnitLists.ENGLISH_SHIP_PREFIX)
        },
    "CIVILIZATION_FRANCE": {
        ATTACK_SUBMARINE: ListNamer("FrA", None, NavalUnitLists.FRENCH_ATTACK_SUBMARINES),
        BATTLESHIP: ListNamer("FrB", None, NavalUnitLists.FRENCH_BATTLESHIPS),
        CARAVEL: FRENCH_SHARED_NAMER_1,
        CARRIER: ListNamer("FrC", None, NavalUnitLists.FRENCH_CARRIERS),
        DESTROYER: ListNamer("FrD", None, NavalUnitLists.FRENCH_DESTROYERS),
        FRIGATE: FRENCH_SHARED_NAMER_1,
        GALLEON: ListNamer("FrGn", None, NavalUnitLists.FRENCH_GALLEONS),
        GALLEY: ListNamer("FrGy", None, NavalUnitLists.FRENCH_GALLEYS),
        IRONCLAD: ListNamer("FrI", None, NavalUnitLists.FRENCH_IRONCLADS),
        PRIVATEER: ListNamer("FrP", None, NavalUnitLists.FRENCH_PRIVATEERS),
        SHIP_OF_THE_LINE: ListNamer("FrSl", None, NavalUnitLists.FRENCH_SHIPS_OF_THE_LINE),
        SUBMARINE: ListNamer("FrSu", None, NavalUnitLists.FRENCH_SUBMARINES),
        TRANSPORT: ListNamer("FrT", None, NavalUnitLists.FRENCH_TRANSPORTS),
        TRIREME: FRENCH_SHARED_NAMER_1,
        DEFAULT: Namer(None)
        },
    "CIVILIZATION_GERMANY": {
        ATTACK_SUBMARINE: GERMAN_SUBMARINE_NAMER,
        BATTLESHIP: ListNamer("GeB", NavalUnitLists.GERMAN_SHIP_PREFIX, NavalUnitLists.GERMAN_BATTLESHIPS),
        CARAVEL: GERMAN_SHARED_NAMER_1,
        DESTROYER: GERMAN_SHARED_NAMER_1,
        FRIGATE: GERMAN_SHARED_NAMER_1,
        GALLEON: GERMAN_SHARED_NAMER_2,
        GALLEY: GERMAN_SHARED_NAMER_2,
        IRONCLAD: ListNamer("GeI", NavalUnitLists.GERMAN_SHIP_PREFIX, NavalUnitLists.GERMAN_IRONCLADS),
        PRIVATEER: ListNamer("GeP", None, NavalUnitLists.GERMAN_PRIVATEERS),
        SHIP_OF_THE_LINE: GERMAN_SHARED_NAMER_1,
        SUBMARINE: GERMAN_SUBMARINE_NAMER,
        TRANSPORT: ListNamer("GeT", NavalUnitLists.GERMAN_SHIP_PREFIX, NavalUnitLists.GERMAN_TRANSPORTS),
        TRIREME: GERMAN_SHARED_NAMER_1,
        DEFAULT: Namer(NavalUnitLists.GERMAN_SHIP_PREFIX)
        },
    "CIVILIZATION_GREECE": {
        ATTACK_SUBMARINE: GREEK_SHARED_NAMER_2,
        BATTLESHIP: ListNamer("GrB", NavalUnitLists.GREEK_SHIP_PREFIX, NavalUnitLists.GREEK_BATTLESHIPS),
        CARAVEL: GREEK_SHARED_NAMER_1,
        DESTROYER: GREEK_SHARED_NAMER_1,
        FRIGATE: GREEK_SHARED_NAMER_1,
        GALLEON: GREEK_SHARED_NAMER_3,
        GALLEY: GREEK_SHARED_NAMER_3,
        IRONCLAD: GREEK_SHARED_NAMER_1,
        PRIVATEER: ListNamer("GrP", None, NavalUnitLists.GREEK_PRIVATEERS),
        SHIP_OF_THE_LINE: GREEK_SHARED_NAMER_1,
        SUBMARINE: GREEK_SHARED_NAMER_2,
        TRANSPORT: GREEK_SHARED_NAMER_3,
        TRIREME: GREEK_SHARED_NAMER_1,
        DEFAULT: Namer(NavalUnitLists.GREEK_SHIP_PREFIX)
        },
    "CIVILIZATION_INDIA": {
        ATTACK_SUBMARINE: INDIAN_SHARED_NAMER_3,
        CARAVEL: INDIAN_SHARED_NAMER_1,
        CARRIER: ListNamer("InC", NavalUnitLists.INDIAN_SHIP_PREFIX, NavalUnitLists.INDIAN_CARRIERS),
        DESTROYER: INDIAN_SHARED_NAMER_1,
        FRIGATE: INDIAN_SHARED_NAMER_1,
        GALLEON: INDIAN_SHARED_NAMER_2,
        GALLEY: INDIAN_SHARED_NAMER_2,
        IRONCLAD: INDIAN_SHARED_NAMER_1,
        SHIP_OF_THE_LINE: INDIAN_SHARED_NAMER_1,
        SUBMARINE: INDIAN_SHARED_NAMER_3,
        TRANSPORT: INDIAN_SHARED_NAMER_2,
        TRIREME: INDIAN_SHARED_NAMER_1,
        DEFAULT: Namer(NavalUnitLists.INDIAN_SHIP_PREFIX)
        },
    "CIVILIZATION_JAPAN": {
        ATTACK_SUBMARINE: JAPANESE_SUBMARINE_NAMER,
        BATTLESHIP: ListNamer("JpB", None, NavalUnitLists.JAPANESE_BATTLESHIPS),
        CARAVEL: JAPANESE_SHARED_NAMER_1,
        CARRIER: ListNamer("JpC", None, NavalUnitLists.JAPANESE_CARRIERS),
        DESTROYER: JAPANESE_SHARED_NAMER_3,
        FRIGATE: JAPANESE_SHARED_NAMER_1,
        GALLEON: JAPANESE_SHARED_NAMER_2,
        GALLEY: JAPANESE_SHARED_NAMER_2,
        IRONCLAD: JAPANESE_SHARED_NAMER_3,
        SHIP_OF_THE_LINE: JAPANESE_SHARED_NAMER_3,
        SUBMARINE: JAPANESE_SUBMARINE_NAMER,
        TRANSPORT: JAPANESE_SHARED_NAMER_2,
        TRIREME: JAPANESE_SHARED_NAMER_1,
        DEFAULT: Namer(None)
        },
    "CIVILIZATION_KOREA": {
        ATTACK_SUBMARINE: KOREAN_SHARED_NAMER_1,
        CARAVEL: KOREAN_SHARED_NAMER_1,
        DESTROYER: KOREAN_SHARED_NAMER_2,
        FRIGATE: KOREAN_SHARED_NAMER_2,
        GALLEON: KOREAN_SHARED_NAMER_3,
        GALLEY: KOREAN_SHARED_NAMER_3,
        IRONCLAD: KOREAN_SHARED_NAMER_2,
        SHIP_OF_THE_LINE: KOREAN_SHARED_NAMER_2,
        SUBMARINE: KOREAN_SHARED_NAMER_1,
        TRANSPORT: KOREAN_SHARED_NAMER_3,
        TRIREME: KOREAN_SHARED_NAMER_1,
        DEFAULT: Namer(NavalUnitLists.KOREAN_SHIP_PREFIX)
        },
    "CIVILIZATION_NETHERLANDS": {
        ATTACK_SUBMARINE: DUTCH_SHARED_NAMER_3,
        BATTLESHIP: ListNamer("DuB", NavalUnitLists.DUTCH_SHIP_PREFIX, NavalUnitLists.DUTCH_BATTLESHIPS),
        CARAVEL: DUTCH_SHARED_NAMER_1,
        CARRIER: ListNamer("DuC", NavalUnitLists.DUTCH_SHIP_PREFIX, NavalUnitLists.DUTCH_CARRIERS),
        DESTROYER: ListNamer("DuD", NavalUnitLists.DUTCH_SHIP_PREFIX, NavalUnitLists.DUTCH_DESTROYERS),
        "East Indiaman": ListNamer("DuE", NavalUnitLists.DUTCH_EAST_INDIA_COMPANY_PREFIX, NavalUnitLists.DUTCH_EAST_INDIAMEN),
        FRIGATE: DUTCH_SHARED_NAMER_1,
        GALLEY: DUTCH_SHARED_NAMER_2,
        IRONCLAD: ListNamer("DuI", NavalUnitLists.DUTCH_SHIP_PREFIX, NavalUnitLists.DUTCH_IRONCLADS),
        PRIVATEER: ListNamer("DuP", None, NavalUnitLists.DUTCH_PRIVATEERS),
        SHIP_OF_THE_LINE: ListNamer("DuSl", NavalUnitLists.DUTCH_SHIP_PREFIX, NavalUnitLists.DUTCH_SHIPS_OF_THE_LINE),
        SUBMARINE: DUTCH_SHARED_NAMER_3,
        TRANSPORT: DUTCH_SHARED_NAMER_2,
        TRIREME: DUTCH_SHARED_NAMER_1,
        DEFAULT: Namer(NavalUnitLists.DUTCH_SHIP_PREFIX)
        },
    "CIVILIZATION_OTTOMAN": {
        ATTACK_SUBMARINE: ListNamer("OtA", NavalUnitLists.OTTOMAN_SHIP_PREFIX, NavalUnitLists.OTTOMAN_ATTACK_SUBMARINES),
        BATTLESHIP: ListNamer("OtB", NavalUnitLists.OTTOMAN_SHIP_PREFIX, NavalUnitLists.OTTOMAN_BATTLESHIPS),
        CARAVEL: OTTOMAN_SHARED_NAMER_1,
        DESTROYER: ListNamer("OtD", NavalUnitLists.OTTOMAN_SHIP_PREFIX, NavalUnitLists.OTTOMAN_DESTROYERS),
        FRIGATE: OTTOMAN_SHARED_NAMER_1,
        GALLEON: OTTOMAN_SHARED_NAMER_2,
        GALLEY: OTTOMAN_SHARED_NAMER_2,
        IRONCLAD: ListNamer("OtI", NavalUnitLists.OTTOMAN_SHIP_PREFIX, NavalUnitLists.OTTOMAN_IRONCLADS),
        PRIVATEER: ListNamer("OtP", None, NavalUnitLists.OTTOMAN_PRIVATEERS),
        SHIP_OF_THE_LINE: ListNamer("OtSl", NavalUnitLists.OTTOMAN_SHIP_PREFIX, NavalUnitLists.OTTOMAN_SHIPS_OF_THE_LINE),
        TRANSPORT: OTTOMAN_SHARED_NAMER_2,
        TRIREME: OTTOMAN_SHARED_NAMER_1,
        DEFAULT: Namer(NavalUnitLists.OTTOMAN_SHIP_PREFIX)
        },
    "CIVILIZATION_PERSIA": {
        ATTACK_SUBMARINE: ListNamer("PrA", None, NavalUnitLists.PERSIAN_ATTACK_SUBMARINES),
        CARAVEL: PERSIAN_SHARED_NAMER_1,
        DESTROYER: PERSIAN_SHARED_NAMER_1,
        FRIGATE: PERSIAN_SHARED_NAMER_1,
        GALLEY: PERSIAN_SHARED_NAMER_2,
        GALLEON: PERSIAN_SHARED_NAMER_2,
        IRONCLAD: PERSIAN_SHARED_NAMER_1,
        SHIP_OF_THE_LINE: PERSIAN_SHARED_NAMER_1,
        TRANSPORT: PERSIAN_SHARED_NAMER_2,
        TRIREME: PERSIAN_SHARED_NAMER_1,
        DEFAULT: Namer(None)
        },
    "CIVILIZATION_PORTUGAL": {
        ATTACK_SUBMARINE: ListNamer("PoA", NavalUnitLists.PORTUGUESE_SHIP_PREFIX, NavalUnitLists.PORTUGUESE_ATTACK_SUBMARINES),
        "Carrack": PORTUGUESE_SHARED_NAMER_2,
        FRIGATE: ListNamer("PoF", NavalUnitLists.PORTUGUESE_SHIP_PREFIX, NavalUnitLists.PORTUGUESE_FRIGATES),
        GALLEON: PORTUGUESE_SHARED_NAMER_1,
        GALLEY: PORTUGUESE_SHARED_NAMER_1,
        SHIP_OF_THE_LINE: ListNamer("PoSl", NavalUnitLists.PORTUGUESE_SHIP_PREFIX, NavalUnitLists.PORTUGUESE_SHIPS_OF_THE_LINE),
        TRIREME: PORTUGUESE_SHARED_NAMER_2,
        DEFAULT: Namer(NavalUnitLists.PORTUGUESE_SHIP_PREFIX)
        },
    "CIVILIZATION_RUSSIA": {
        ATTACK_SUBMARINE: ListNamer("RuA", None, NavalUnitLists.RUSSIAN_ATTACK_SUBMARINES),
        BATTLESHIP: ListNamer("RuB", None, NavalUnitLists.RUSSIAN_BATTLESHIPS),
        CARAVEL: RUSSIAN_SHARED_NAMER_1,
        CARRIER: ListNamer("RuC", None, NavalUnitLists.RUSSIAN_CARRIERS),
        DESTROYER: ListNamer("RuD", None, NavalUnitLists.RUSSIAN_DESTROYERS),
        FRIGATE: RUSSIAN_SHARED_NAMER_1,
        GALLEON: RUSSIAN_SHARED_NAMER_2,
        GALLEY: RUSSIAN_SHARED_NAMER_2,
        IRONCLAD: ListNamer("RuI", None, NavalUnitLists.RUSSIAN_IRONCLADS),
        MISSILE_CRUISER: ListNamer("RuMC", None, NavalUnitLists.RUSSIAN_MISSILE_CRUISERS),
        SHIP_OF_THE_LINE: ListNamer("RuSL", None, NavalUnitLists.RUSSIAN_SHIPS_OF_THE_LINE),
        SUBMARINE: IncreasingSequenceNamer("RuS", NavalUnitLists.RUSSIAN_SUBMARINE_PREFIX, 180, 21),
        TRANSPORT: RUSSIAN_SHARED_NAMER_2,
        TRIREME: RUSSIAN_SHARED_NAMER_1,
        DEFAULT: Namer(None)
        },
    "CIVILIZATION_SPAIN": {
        ATTACK_SUBMARINE: ListNamer("SpA", None, NavalUnitLists.SPANISH_ATTACK_SUBMARINES),
        BATTLESHIP: ListNamer("SpB", None, NavalUnitLists.SPANISH_BATTLESHIPS),
        CARAVEL: SPANISH_SHARED_NAMER_2,
        CARRIER: ListNamer("SpAC", None, NavalUnitLists.SPANISH_CARRIERS),
        DESTROYER: ListNamer("SpD", None, NavalUnitLists.SPANISH_DESTROYERS),
        FRIGATE: ListNamer("SpF", None, NavalUnitLists.SPANISH_FRIGATES),
        GALLEON: SPANISH_SHARED_NAMER_1,
        GALLEY: SPANISH_SHARED_NAMER_1,
        IRONCLAD: ListNamer("SpI", None, NavalUnitLists.SPANISH_IRONCLADS),
        PRIVATEER: ListNamer("SpP", None, NavalUnitLists.SPANISH_PRIVATEERS),
        SHIP_OF_THE_LINE: ListNamer("SpSl", None, NavalUnitLists.SPANISH_SHIPS_OF_THE_LINE),
        SUBMARINE: ListNamer("SpS", None, NavalUnitLists.SPANISH_SUBMARINES),
        TRANSPORT: ListNamer("SpT", None, NavalUnitLists.SPANISH_TRANSPORTS),
        TRIREME: SPANISH_SHARED_NAMER_2,
        DEFAULT: Namer(None)
        }
    }

DEFAULT_NAMER = Namer(None)

def getNamer(iCivilization, unitClass):
    info = gc.getCivilizationInfo(iCivilization)
    strCivilizationType = info.getType()
    if strCivilizationType in NAMER_MAP:
        namers = NAMER_MAP[strCivilizationType]
        if unitClass in namers:
            return namers[unitClass]
        elif DEFAULT in namers:
            return namers[DEFAULT]
    return DEFAULT_NAMER

def generateName(pattern, iCivilization, unitClass, count):
    prefixOverride = None
    p = re.compile('^\^nav:(.*)\^$')
    m = p.match(pattern)
    if (m is not None):
        prefixOverride = m.group(1)
    namer = getNamer(iCivilization, unitClass)
    return namer.generateWithPrefix(prefixOverride, iCivilization, unitClass, count)
